# main
__all__ = ['Authentication']

# Explicit imports
from .authentication import Authentication


# import os
# import sys
# 
# # Absolute path of this package's directory
# PACKAGEPATH = os.path.abspath(os.path.dirname(__file__))
# 
# # Add the package path to sys.path
# if PACKAGEPATH not in sys.path:
#     sys.path.append(PACKAGEPATH)
# 
# # # Define HOMEPATH for general usage
# # HOMEPATH = os.path.dirname(PACKAGEPATH)
# 
# from authentication import Authentication
# 
# __all__ = ['Authentication']
